@@ -52,6 +52,8 @@ static struct {
 		DBUS_TYPE_STRING, "IPv4.Netmask" },
 	{ CONNMAN_PROPERTY_ID_IPV4_GATEWAY,
 		DBUS_TYPE_STRING, "IPv4.Gateway" },
+	{ CONNMAN_PROPERTY_ID_IPV4_BROADCAST,
+		DBUS_TYPE_STRING, "IPv4.Broadcast" },
 	{ CONNMAN_PROPERTY_ID_IPV4_NAMESERVER,
 		DBUS_TYPE_STRING, "IPv4.Nameserver" },
 
@@ -143,6 +145,28 @@ static const char *subtype2string(enum connman_element_subtype type)
 	return NULL;
 }
 
+static const char *subtype2description(enum connman_element_subtype type)
+{
+	switch (type) {
+	case CONNMAN_ELEMENT_SUBTYPE_UNKNOWN:
+	case CONNMAN_ELEMENT_SUBTYPE_FAKE:
+	case CONNMAN_ELEMENT_SUBTYPE_NETWORK:
+		return NULL;
+	case CONNMAN_ELEMENT_SUBTYPE_ETHERNET:
+		return "Ethernet";
+	case CONNMAN_ELEMENT_SUBTYPE_WIFI:
+		return "Wireless";
+	case CONNMAN_ELEMENT_SUBTYPE_WIMAX:
+		return "WiMAX";
+	case CONNMAN_ELEMENT_SUBTYPE_MODEM:
+		return "Modem";
+	case CONNMAN_ELEMENT_SUBTYPE_BLUETOOTH:
+		return "Bluetooth";
+	}
+
+	return NULL;
+}
+
 const char *__connman_element_policy2string(enum connman_element_policy policy)
 {
 	switch (policy) {
@@ -328,6 +352,10 @@ static void emit_enabled_signal(DBusConnection *conn,
 		iface = CONNMAN_NETWORK_INTERFACE;
 		key = "Connected";
 		break;
+	case CONNMAN_ELEMENT_TYPE_CONNECTION:
+		iface = CONNMAN_CONNECTION_INTERFACE;
+		key = "Default";
+		break;
 	default:
 		return;
 	}
@@ -350,6 +378,39 @@ static void emit_enabled_signal(DBusConnection *conn,
 	g_dbus_send_message(conn, signal);
 }
 
+static void emit_scanning_signal(DBusConnection *conn,
+					struct connman_element *element)
+{
+	DBusMessage *signal;
+	DBusMessageIter entry, value;
+	const char *key = "Scanning";
+
+	DBG("conn %p", conn);
+
+	if (element == NULL)
+		return;
+
+	if (element->type != CONNMAN_ELEMENT_TYPE_DEVICE)
+		return;
+
+	signal = dbus_message_new_signal(element->path,
+				CONNMAN_DEVICE_INTERFACE, "PropertyChanged");
+	if (signal == NULL)
+		return;
+
+	dbus_message_iter_init_append(signal, &entry);
+
+	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
+
+	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
+					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
+	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN,
+							&element->scanning);
+	dbus_message_iter_close_container(&entry, &value);
+
+	g_dbus_send_message(conn, signal);
+}
+
 static DBusMessage *do_update(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
@@ -459,6 +520,15 @@ static DBusMessage *device_get_properties(DBusConnection *conn,
 			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
 			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
 
+	str = subtype2description(element->subtype);
+	if (str != NULL && element->devname != NULL) {
+		char *name = g_strdup_printf("%s (%s)", str, element->devname);
+		if (name != NULL)
+			connman_dbus_dict_append_variant(&dict, "Name",
+						DBUS_TYPE_STRING, &name);
+		g_free(name);
+	}
+
 	str = subtype2string(element->subtype);
 	if (str != NULL)
 		connman_dbus_dict_append_variant(&dict, "Type",
@@ -473,7 +543,11 @@ static DBusMessage *device_get_properties(DBusConnection *conn,
 					DBUS_TYPE_BOOLEAN, &element->enabled);
 
 	if (element->subtype == CONNMAN_ELEMENT_SUBTYPE_WIFI ||
-			element->subtype == CONNMAN_ELEMENT_SUBTYPE_WIMAX) {
+			element->subtype == CONNMAN_ELEMENT_SUBTYPE_WIMAX ||
+			element->subtype == CONNMAN_ELEMENT_SUBTYPE_BLUETOOTH) {
+		connman_dbus_dict_append_variant(&dict, "Scanning",
+					DBUS_TYPE_BOOLEAN, &element->scanning);
+
 		dbus_message_iter_open_container(&dict, DBUS_TYPE_DICT_ENTRY,
 								NULL, &entry);
 		append_networks(element, &entry);
@@ -512,9 +586,9 @@ static DBusMessage *device_set_property(DBusConnection *conn,
 		dbus_message_iter_get_basic(&value, &powered);
 
 		if (powered == TRUE)
-			do_enable(conn, msg, data);
+			do_enable(conn, msg, element);
 		else
-			do_disable(conn, msg, data);
+			do_disable(conn, msg, element);
 	} else
 		set_common_property(element, name, &value);
 
@@ -563,6 +637,9 @@ static DBusMessage *device_create_network(DBusConnection *conn,
 
 	DBG("conn %p", conn);
 
+	if (element->subtype != CONNMAN_ELEMENT_SUBTYPE_WIFI)
+		return __connman_error_invalid_arguments(msg);
+
 	if (dbus_message_iter_init(msg, &iter) == FALSE)
 		return __connman_error_invalid_arguments(msg);
 
@@ -622,6 +699,10 @@ static DBusMessage *network_get_properties(DBusConnection *conn,
 			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
 			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
 
+	if (element->parent)
+		connman_dbus_dict_append_variant(&dict, "Device",
+				DBUS_TYPE_OBJECT_PATH, &element->parent->path);
+
 	str = __connman_element_policy2string(element->policy);
 	if (str != NULL)
 		connman_dbus_dict_append_variant(&dict, "Policy",
@@ -679,12 +760,13 @@ static DBusMessage *network_set_property(DBusConnection *conn,
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
-static DBusMessage *get_connection_properties(DBusConnection *conn,
+static DBusMessage *connection_get_properties(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
 	struct connman_element *element = data;
 	DBusMessage *reply;
 	DBusMessageIter array, dict;
+	const char *str;
 
 	DBG("conn %p", conn);
 
@@ -699,6 +781,19 @@ static DBusMessage *get_connection_properties(DBusConnection *conn,
 			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
 			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
 
+	str = subtype2string(element->subtype);
+	if (str != NULL)
+		connman_dbus_dict_append_variant(&dict, "Type",
+						DBUS_TYPE_STRING, &str);
+
+	if (element->subtype == CONNMAN_ELEMENT_SUBTYPE_WIFI ||
+			element->subtype == CONNMAN_ELEMENT_SUBTYPE_WIMAX)
+		connman_dbus_dict_append_variant(&dict, "Strength",
+					DBUS_TYPE_BYTE, &element->strength);
+
+	connman_dbus_dict_append_variant(&dict, "Default",
+					DBUS_TYPE_BOOLEAN, &element->enabled);
+
 	add_common_properties(element, &dict);
 
 	dbus_message_iter_close_container(&array, &dict);
@@ -706,6 +801,39 @@ static DBusMessage *get_connection_properties(DBusConnection *conn,
 	return reply;
 }
 
+static DBusMessage *connection_set_property(DBusConnection *conn,
+					DBusMessage *msg, void *data)
+{
+	struct connman_element *element = data;
+	DBusMessageIter iter, value;
+	const char *name;
+
+	DBG("conn %p", conn);
+
+	if (dbus_message_iter_init(msg, &iter) == FALSE)
+		return __connman_error_invalid_arguments(msg);
+
+	dbus_message_iter_get_basic(&iter, &name);
+	dbus_message_iter_next(&iter);
+	dbus_message_iter_recurse(&iter, &value);
+
+	if (__connman_security_check_privileges(msg) < 0)
+		return __connman_error_permission_denied(msg);
+
+	if (g_str_equal(name, "Default") == TRUE) {
+		dbus_bool_t enabled;
+
+		dbus_message_iter_get_basic(&value, &enabled);
+
+		if (enabled == TRUE)
+			return do_enable(conn, msg, element);
+		else
+			return do_disable(conn, msg, element);
+	}
+
+	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
+}
+
 static GDBusMethodTable device_methods[] = {
 	{ "GetProperties", "",      "a{sv}", device_get_properties },
 	{ "SetProperty",   "sv",    "",      device_set_property   },
@@ -724,7 +852,8 @@ static GDBusMethodTable network_methods[] = {
 };
 
 static GDBusMethodTable connection_methods[] = {
-	{ "GetProperties", "",   "a{sv}", get_connection_properties },
+	{ "GetProperties", "",   "a{sv}", connection_get_properties },
+	{ "SetProperty",   "sv", "",      connection_set_property   },
 	{ },
 };
 
@@ -1128,6 +1257,9 @@ static void emit_property_changed(DBusConnection *conn,
 	case CONNMAN_ELEMENT_TYPE_NETWORK:
 		iface = CONNMAN_NETWORK_INTERFACE;
 		break;
+	case CONNMAN_ELEMENT_TYPE_CONNECTION:
+		iface = CONNMAN_CONNECTION_INTERFACE;
+		break;
 	default:
 		return;
 	}
@@ -1422,6 +1554,12 @@ int connman_element_set_property(struct connman_element *element,
 		element->ipv4.gateway = g_strdup(*((const char **) value));
 		__connman_element_unlock(element);
 		break;
+	case CONNMAN_PROPERTY_ID_IPV4_BROADCAST:
+		__connman_element_lock(element);
+		g_free(element->ipv4.broadcast);
+		element->ipv4.broadcast = g_strdup(*((const char **) value));
+		__connman_element_unlock(element);
+		break;
 	case CONNMAN_PROPERTY_ID_IPV4_NAMESERVER:
 		__connman_element_lock(element);
 		g_free(element->ipv4.nameserver);
@@ -1478,6 +1616,14 @@ int connman_element_get_value(struct connman_element *element,
 		*((char **) value) = element->ipv4.gateway;
 		__connman_element_unlock(element);
 		break;
+	case CONNMAN_PROPERTY_ID_IPV4_BROADCAST:
+		if (element->ipv4.broadcast == NULL)
+			return connman_element_get_value(element->parent,
+								id, value);
+		__connman_element_lock(element);
+		*((char **) value) = element->ipv4.broadcast;
+		__connman_element_unlock(element);
+		break;
 	case CONNMAN_PROPERTY_ID_IPV4_NAMESERVER:
 		if (element->ipv4.nameserver == NULL)
 			return connman_element_get_value(element->parent,
@@ -1733,6 +1879,20 @@ static void emit_state_change(DBusConnection *conn, const char *state)
 	g_dbus_send_message(conn, signal);
 }
 
+static void set_signal_strength(struct connman_element *connection)
+{
+	struct connman_element *element = connection;
+
+	while (element != NULL) {
+		if (element->type == CONNMAN_ELEMENT_TYPE_NETWORK) {
+			connection->strength = element->strength;
+			break;
+		}
+
+		element = element->parent;
+	}
+}
+
 static void register_element(gpointer data, gpointer user_data)
 {
 	struct connman_element *element = data;
@@ -1799,6 +1959,7 @@ static void register_element(gpointer data, gpointer user_data)
 			connman_error("Failed to register %s connection",
 								element->path);
 		else {
+			set_signal_strength(element);
 			emit_connections_signal(connection);
 			emit_state_change(connection, "online");
 		}
@@ -1902,7 +2063,9 @@ static gboolean remove_element(GNode *node, gpointer user_data)
 	}
 
 	if (element->type == CONNMAN_ELEMENT_TYPE_CONNECTION) {
-		emit_state_change(connection, "offline");
+		if (__connman_element_count(NULL,
+					CONNMAN_ELEMENT_TYPE_CONNECTION) == 0)
+			emit_state_change(connection, "offline");
 		emit_connections_signal(connection);
 
 		g_dbus_unregister_interface(connection, element->path,
@@ -1958,12 +2121,22 @@ void connman_element_unregister_children(struct connman_element *element)
 static gboolean update_element(GNode *node, gpointer user_data)
 {
 	struct connman_element *element = node->data;
+	struct connman_element *root = user_data;
 
 	DBG("element %p name %s", element, element->name);
 
 	if (element->driver && element->driver->update)
 		element->driver->update(element);
 
+	if (element->type == CONNMAN_ELEMENT_TYPE_CONNECTION &&
+				root->type == CONNMAN_ELEMENT_TYPE_NETWORK) {
+		if (element->strength != root->strength) {
+			element->strength = root->strength;
+			emit_property_changed(connection, element, "Strength",
+					DBUS_TYPE_BYTE, &element->strength);
+		}
+	}
+
 	return FALSE;
 }
 
@@ -1977,7 +2150,7 @@ void connman_element_update(struct connman_element *element)
 
 	if (node != NULL)
 		g_node_traverse(node, G_PRE_ORDER,
-				G_TRAVERSE_ALL, -1, update_element, NULL);
+				G_TRAVERSE_ALL, -1, update_element, element);
 }
 
 int connman_element_set_enabled(struct connman_element *element,
@@ -1993,6 +2166,19 @@ int connman_element_set_enabled(struct connman_element *element,
 	return 0;
 }
 
+int connman_element_set_scanning(struct connman_element *element,
+							gboolean scanning)
+{
+	if (element->scanning == scanning)
+		return 0;
+
+	element->scanning = scanning;
+
+	emit_scanning_signal(connection, element);
+
+	return 0;
+}
+
 int __connman_element_init(DBusConnection *conn, const char *device)
 {
 	struct connman_element *element;
